import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

/**
 * @author David Foster <davidfstr@gmail.com>
 */
public class GrayscaleLevelHistogram implements PixelBufferToBufferedImageFilter, ParameterizedFilter {
	private final static int HISTOGRAM_BIN_WIDTH = 1;
	
	private final static int HISTOGRAM_WIDTH = HISTOGRAM_BIN_WIDTH * 256;
	private final static int HISTOGRAM_HEIGHT = 150;
	
	private final static Color BKGND_COLOR = Color.WHITE;
	private final static Color FRGND_COLOR = new Color(255*1/2, 255*1/2, 255);
	private final static Color BORDER_COLOR = Color.BLACK;
	
	private final static Color[] BIN_COLORS;
	static {
		BIN_COLORS = new Color[256];
		for (int i=0; i<256; i++) {
			BIN_COLORS[i] = new Color(i, i, i);
		}
	}
	
	private final FloatFilterParam binHeightScaleFactorParam =
		new FloatFilterParam("Bin Height Scale Factor", 0.00f, Float.MAX_VALUE, 1.00f);
	
	private final FilterParam[] params = new FilterParam[] {
		binHeightScaleFactorParam
	};
	
	// ### FILTER ###
	
	private int[] levels = new int[256];
	
	public void filter(int width, int height, int[] inPixels,
	                   BufferedImage inImg, BufferedImage outImg)
	{
		float binHeightScaleFactor = binHeightScaleFactorParam.getValue();
		
		int numPixels = inPixels.length;
		
		/* 
		 * Obtain histogram of the frequencies of the 
		 * different levels of gray in the input image
		 */
		int maxLevel = 0;
		{
			for (int bin=0; bin<256; bin++) {
				levels[bin] = 0;
			}
			
			for (int i=0, n=numPixels; i<n; i++) {
				levels[pixelToLevel(inPixels[i])]++;
			}
			
			for (int bin=0; bin<256; bin++) {
				int curLevel = levels[bin];
				if (curLevel > maxLevel) {
					maxLevel = curLevel;
				}
			}
		}
		
		/* Draw the histogram */
		{
			Graphics2D g = (Graphics2D) outImg.getGraphics();
			
			/* Fill output image with background color behind histogram */
			g.setColor(BKGND_COLOR);
			g.fillRect(0, 0, width, height);
			
			/* 
			 * Translate coordinate space to the upper-left corner of
			 * the histogram (which is centered in the middle of the output image)
			 */
			g.translate(
				(width - HISTOGRAM_WIDTH)/2,
				(height - HISTOGRAM_HEIGHT)/2);
			
			/* Draw border around histogram */
			g.setColor(BORDER_COLOR);
			g.drawRect(-1, -1, HISTOGRAM_WIDTH+1, HISTOGRAM_HEIGHT+1);
			
			/* Draw foreground of histogram */
			g.setColor(FRGND_COLOR);
			g.fillRect(0, 0, HISTOGRAM_WIDTH, HISTOGRAM_HEIGHT);
			
			/* Clip subsequent drawing inside the histogram region */
			g.setClip(0, 0, HISTOGRAM_WIDTH, HISTOGRAM_HEIGHT);
			
			/* Draw the levels inside the histogram */
			int binX=0;
			for (int bin=0; bin<256; bin++, binX+=HISTOGRAM_BIN_WIDTH) {
				int curLevel = levels[bin];
				int curBinHeight = (int) (binHeightScaleFactor*(curLevel*HISTOGRAM_HEIGHT)/maxLevel);
				Color curBinColor = BIN_COLORS[bin];
				
				// Draw the current bin
				g.setColor(curBinColor);
				g.fillRect(
					binX, HISTOGRAM_HEIGHT-curBinHeight,
					HISTOGRAM_BIN_WIDTH, curBinHeight);
			}
		}
	}
	
	private static int pixelToLevel(int pixel) {
		return (Pixels.getRed(pixel) + Pixels.getGreen(pixel) + Pixels.getBlue(pixel))/3;
	}
	
	public String toString() {
		return "Grayscale Level Histogram";
	}
	
	// ### PARAMETERIZED FILTER ###
	
	public FilterParam[] getParams() {
		return params;
	}
}